import uuid
from decimal import Decimal
from ool import VersionField, VersionedMixin
from django.core.validators import MinValueValidator
from django.db import models


class TicketBase(VersionedMixin, models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    title = models.CharField('Title', max_length=50)
    price = models.DecimalField('Price', max_digits=10, decimal_places=4,
                                validators=[MinValueValidator(Decimal('0.01'))])
    version = VersionField()

    class Meta:
        abstract = True



import boto3
import json
import pika
from abc import ABC, abstractclassmethod
from django.conf import settings


class PublisherBase(ABC):

    @abstractclassmethod
    def publish(cls, topic_name, body):
        pass


class Sqs(PublisherBase):

    @classmethod
    def publish(cls, topic_name, body):
        message_attributes = {
            'routing_key': {
                'StringValue': topic_name,
                'DataType': 'String'
            }
        }
        sqs = boto3.resource('sqs', region_name=settings.AWS_REGION_NAME, aws_access_key_id=settings.AWS_ACCESS_KEY_ID,
                             aws_secret_access_key=settings.AWS_SECRET_ACCESS_KEY)
        queue = sqs.get_queue_by_name(QueueName='ticketo')
        queue.send_message(MessageBody=json.dumps(body), MessageAttributes=message_attributes)


class RabbitMQ(PublisherBase):

    @classmethod
    def publish(cls, topic_name, body):
        print(settings.RABBITMQ_URL)
        params = pika.URLParameters(settings.RABBITMQ_URL)
        connection = pika.BlockingConnection(params)
        channel = connection.channel()
        channel.exchange_declare(exchange=topic_name, exchange_type='topic', durable=True)
        channel.basic_publish(exchange=topic_name, routing_key=topic_name, body=json.dumps(body))


class Publisher:
    SQS = 'SQS'
    RABBITMQ = 'RABBITMQ'

    @staticmethod
    def publish(topic_name, body):
        publishers = {
            Publisher.SQS: Sqs(),
            Publisher.RABBITMQ: RabbitMQ()
        }
        publisher = publishers[settings.BROKER_NAME]
        publisher.publish(topic_name, body)



import boto3
import pika
from abc import ABC, abstractclassmethod
from django.conf import settings


class ExchangeBase(ABC):

    @abstractclassmethod
    def get_exchange_methods():
        pass


class ConsumerBase(ABC):

    @abstractclassmethod
    def __init__():
        pass

    @abstractclassmethod
    def consumer_fn():
        pass

    @abstractclassmethod
    def consume(cls):
        pass


class Sqs(ConsumerBase):

    def __init__(self, exchanges):
        self.exchanges = exchanges

    def consumer_fn():
        pass

    def consume(self):
        sqs = boto3.resource('sqs', region_name=settings.AWS_REGION_NAME, aws_access_key_id=settings.AWS_ACCESS_KEY_ID,
                             aws_secret_access_key=settings.AWS_SECRET_ACCESS_KEY)
        queue = sqs.get_queue_by_name(QueueName='ticketo')
        for message in queue.receive_messages(MessageAttributeNames=['routing_key']):
            # QUEUES = {
            #     'ticketo_ticket_created': self.ticket_created,
            #     'ticketo_ticket_updated': self.ticket_updated,
            #     'ticketo_ticket_deleted': self.ticket_deleted
            # }
            # queue_fn = QUEUES[header_frame.content_type]
            # queue_fn(body)
            # print(message.message_attributes)
            pass


class RabbitMQ(ConsumerBase):

    def __init__(self, exchanges):
        self.exchanges = exchanges

    def consumer_fn(self, channel, method_frame, header_frame, body):
        exchange_fn = self.exchanges[method_frame.exchange]
        exchange_fn(body)

    def consume(self):
        params = pika.URLParameters(settings.RABBITMQ_URL)
        connection = pika.BlockingConnection(params)
        channel = connection.channel()
        channel.queue_declare(queue=settings.SERVICE_QUEUE, durable=True)
        channel.basic_consume(queue=settings.SERVICE_QUEUE, on_message_callback=self.consumer_fn, auto_ack=True)
        channel.start_consuming()
        channel.close()


class ConsumerFactory:

    @staticmethod
    def get_consumer(consumer_name, exchanges):
        CONSUMERS = {
            'SQS': Sqs,
            'RABBITMQ': RabbitMQ
        }
        Consumer = CONSUMERS[consumer_name]
        return Consumer(exchanges=exchanges)
